/*
   A simple example of communication from the Launchpad to the computer:
  the value of analog input 0 is sent out the serial port. We call this "serial"
  communication because the connection appears to both the Launchpad and the
  computer as a serial port, even though it may actually use
  a USB cable. Bytes are sent one after another (serially) from the Launchpad
  to the computer.

  You can use the Energia serial monitor to view the sent data, or it can
  be read by Processing or another program capable of reading
  data from a serial port. The Processing code below graphs the data received
  so you can see the value of the analog input changing over time.

  The circuit:
  Any analog input sensor can be attached to an analog pin.
  For this example, a potentiometer is attached to analog pin 3.

  created 2006
  by David A. Mellis
  modified 9 Apr 2012
  by Tom Igoe and Scott Fitzgerald
  Modified 15 April 2013
  By Sean Alvarado
  Modified  July 19, 2017
  by Brian Cruickshank - Serial I/O interface to GCv2

  This example code is in the public domain.
 */

#define NUMBER 200
#define MAX_JSON_STRLEN 40
#define NUM_VALUES 10
int greenLedState = HIGH;
int redLedState = LOW;
uint8_t i = 0;
uint32_t loopCtr = 0;
char jsonStrFromHost[MAX_JSON_STRLEN];
char badStrFromHost[MAX_JSON_STRLEN];
uint8_t numBytesRead = 0;
uint8_t j = 0;
uint8_t n = 0;
int rxStrIndex = 0;
int badStrIndex = 0;
int numBadStrings = 0;
boolean toggle = false;
boolean cmdReceived = false;
boolean flag = true;
boolean running = true;
boolean msgOk = false;
void setup() {
    pinMode(RED_LED, OUTPUT);
    pinMode(GREEN_LED, OUTPUT);
    Serial.setTimeout(6000); // 1s timeout when reading from serial port
    Serial.begin(9600);
    pinMode(PUSH2, INPUT_PULLUP);
    greenLedState = HIGH;
    redLedState = LOW;
    digitalWrite(GREEN_LED, greenLedState);
    digitalWrite(RED_LED, redLedState);
    clearStr(jsonStrFromHost);
    clearStr(badStrFromHost);
    rxStrIndex = 0;

    Serial.flush();
    j=0;
}

// For debugging purposes: if you get bad strings from the host,
// the last bad string will be copied into the badStrFromHost character array
void copyBadStr(){
    int m = 0;
    clearStr(badStrFromHost);

    for (m=0; m < MAX_JSON_STRLEN; m++){
        badStrFromHost[m] = jsonStrFromHost[m];
    }
    badStrIndex = rxStrIndex;
    rxStrIndex = 0;
}

void clearStr(char* pBuffer){
    int k = 0;
    for (k=0; k < MAX_JSON_STRLEN; k++){
        pBuffer[k] = (char)0;
    }
}
int getSerialString(){
    char rxChar;
    boolean done = false;
    int waitCtr = 0;
    int numCharsRx = 0;
    int numAvailable = 0;
    int index = 0;
    while (!done){
        numAvailable = Serial.available();
        if (numAvailable > 0) {
            for (index=0; index < numAvailable; index++){
                waitCtr = 0;
                rxChar = Serial.read();
                if ((rxChar != -1) && (rxChar != (char)0)){
                    if (rxChar == (char)10){
                        done = true;
                        numCharsRx = rxStrIndex;
                        rxStrIndex = 0;
                    } else {
                        if (rxStrIndex < MAX_JSON_STRLEN){
                            jsonStrFromHost[rxStrIndex++] = rxChar;
                        } else {
                            done = true;
                            break;
                        }
                    }
                }
            }
            Serial.flush();
        } else {
            if (rxStrIndex == 0) {
                done = true;
                Serial.flush();
            } else {
                // If a partial command has been received
                // keep waiting until the /n delimiter is received
                // or we timeout
                delayMicroseconds(50);     // pauses for 0.05 milliseconds
                waitCtr++;
                if (waitCtr > 2000) {  // timeout after 1 second
                    clearStr(jsonStrFromHost);
                    rxStrIndex = 0;
                    numCharsRx = 0;
                    done = true;
                    Serial.flush();
                }
            }
        }
    }
    return(numCharsRx);

}
void loop() {
    numBytesRead = getSerialString();//Serial.readBytesUntil('\n',jsonStrFromHost,MAX_JSON_STRLEN);
    if (numBytesRead > 2){
        // Data from the host needs to be of the form {"LEDs":{"Red":0,"Green":0}} to turn off
        // the red and green LEDs, {"LEDs":{"Red":1,"Green":1}} to turn on the red and green LEDs.
        // (Code is not generic at all, but it is small  )
        msgOk = false;

        for (int j=2; j < numBytesRead; j++){
            if ((jsonStrFromHost[j] == ':') && (j < (numBytesRead + 1))){
                if (jsonStrFromHost[j-2] == 'd'){
                    msgOk = true;
                    if (jsonStrFromHost[j+1] == '1'){
                        redLedState = HIGH;
                    }  else {
                        redLedState = LOW;
                    }
                    digitalWrite(RED_LED, redLedState);
                    cmdReceived = true;
                    flag=true;
                } else {
                    if (jsonStrFromHost[j-2] == 'n'){
                        msgOk = true;
                        if (jsonStrFromHost[j+1] == '1'){
                            greenLedState = HIGH;
                        }  else {
                            greenLedState = LOW;
                        }
                        digitalWrite(GREEN_LED, greenLedState);
                        cmdReceived = true;
                        flag=true;
                    }
                }
            }
        }
        if (!msgOk){
            // put a breakpoint here
            numBadStrings++;
            copyBadStr();
        } else {
            clearStr(jsonStrFromHost);
            rxStrIndex = 0;
            flag = true;
        }

    } else {
        if (numBytesRead > 0) {
            numBadStrings++;
            copyBadStr();
        }
    }

    // Print average as a JSON string of the form {"temp":99.99,"LEDs":{"Red":0,"Green":1}}\n
    if (flag) {
        Serial.print("{\"count\":");
        Serial.print(loopCtr++, DEC);
        Serial.print(",\"LEDs\":{\"Red\":");
        Serial.print(redLedState, DEC);
        Serial.print(",\"Green\":");
        Serial.print(greenLedState, DEC);
        Serial.print("}}\n");
        flag = false;
        n = 0;
    }


    n++;
    if (n==NUMBER) {
        flag=true;
        n = 0;
    }

    delay(1);
}



